#|
Arxiu: ph_montecarlo.lisp
Contingut: implementacions del métode de Montecarlo per al càlcul d'integrals.
Autor: Felip Alàez Nadal.
Útima actualització: 12-VI-06

|#

#|BSD License

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

   1. Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. |#


(defpackage :ph_montecarlo
  (:use :cl :ph_random)
  (:export #:montecarlo #:uniform_montecarlo)
  (:documentation "This package provides some implementations of the Montecarlo method for evaluating integrals."))

(in-package :ph_montecarlo)

(defun montecarlo ( f interval &key (points 1000) (rand_init #'carlson_init) (rand #'carlson) (seed 95423))
  "Evaluates the integral of f using 'points' points. interval is a cons ( a  b ) , where a < b."
  (let* (
	(sf 0.)
	(sf2 0.)
	(x 0.)
	(a (first interval))
	(b (first (rest interval)))
	(wide (- b a))
	(mitja 0.)
	(error_es 0.)
	aux
	)
    (if (<= b a) (error "Non ordered range. Please, use ( a b), where a < b" )) ;Aquesta condici�d'error ha de ser millorada
    (funcall rand_init seed)
    (dotimes (i points)
      (setq x (+ a (* wide (funcall rand))))
      (setq aux (funcall f x))
      (setq sf (+ sf aux))
      (setq sf2 (+ sf2 (expt aux 2))))
    (setq mitja (* wide (coerce (/ sf points) 'double-float)))
    (setq sf2 (* wide (/ sf2 points)))
    (setq error_es (* wide (coerce  (abs (/ (-  sf2   mitja ) (sqrt points))) 'double-float)))
    (values mitja error_es)))








(defun uniform_montecarlo ( f interval &key (points 1000) )
  "Evaluates the integral of f using 'points' points. interval is a cons ( a  b ) , where a < b. It doesn't use a random number generator. Instead of using It, It uses that if the number of points is big enough, you can aproximate an uniform distribution by a number of points uniformely distributed between 0 and 1, where x_{i} = 1/(N-1)*i . This works because to calculate the integral we don't need the points appear in a aleatorial way, and we win speed. As an example, integrating sin(x)^2 between 0 and pi in my cmucl linux box takes 7 seconds with the conventional method, and 3 with this. Algorithm contributed by David Sanchez i Gregori. "
  (let* (
	(sf 0.)
	(sf2 0.)
	(x 0.)
	(a (first interval))
	(b (first (rest interval)))
	(wide (- b a))
	(mitja 0.)
	(error_es 0.)
	(pas (/ wide (- points 1)))
	aux
	)
    (if (<= b a) (error "Non ordered range. Please, use ( a b), where a < b" )) ;Aquesta condició d'error ha de ser millorada
    (dotimes (i points)
      (setq x (+ x (* i pas)))
      (setq aux (funcall f x))
      (setq sf (+ sf aux))
      (setq sf2 (+ sf2 (expt aux 2.))))
    (setq mitja (* wide (coerce (/ sf points) 'double-float)))
    (setq sf2 (* wide (/ sf2 points)))
    (setq error_es (* wide (coerce  (abs (/ (-  sf2   mitja ) (sqrt points))) 'double-float)))
    (values mitja error_es)))